@@ -36,11 +36,53 @@ def remainder_needed_until_multiple(n, mult):
 def round_down_nearest_multiple(val, mult):
     return (val // mult) * mult
 
+def eval_decorator(fn):
+    def inner(model, *args, **kwargs):
+        was_training = model.training
+        model.eval()
+        out = fn(model, *args, **kwargs)
+        model.train(was_training)
+        return out
+    return inner
+
 # attention related utils
 
 def grad_shrink(t, alpha = 0.1):
     return t * alpha + t.detach() * (1 - alpha)
 
+# sampling helpers
+
+def log(t, eps = 1e-20):
+    return torch.log(t + eps)
+
+def gumbel_noise(t):
+    noise = torch.zeros_like(t).uniform_(0, 1)
+    return -log(-log(noise))
+
+def gumbel_sample(t, temperature = 1., dim = -1):
+    return ((t / temperature) + gumbel_noise(t)).argmax(dim = dim)
+
+def top_k(logits, thres = 0.5):
+    num_logits = logits.shape[-1]
+    k = max(int((1 - thres) * num_logits), 1)
+    val, ind = torch.topk(logits, k)
+    probs = torch.full_like(logits, float('-inf'))
+    probs.scatter_(1, ind, val)
+    return probs
+
+def mask_out_after_eos_id(t, eos_id, mask_value = -1, include_eos = True):
+    eos_mask = (t == eos_id).float()
+
+    if include_eos:
+        eos_mask = F.pad(eos_mask, (1, -1))
+
+    after_eos_mask = eos_mask.cumsum(dim = -1) > 0
+    return t.masked_fill(after_eos_mask, mask_value)
+
+def all_rows_have_eos_id(t, eos_id):
+    eos_mask = (t == eos_id)
+    return torch.any(eos_mask, dim = -1).all()
+
 # classifier free guidance functions
 
 def prob_mask_like(shape, prob, device):
@@ -301,6 +343,81 @@ class SemanticTransformer(nn.Module):
         self.transformer = Transformer(dim = dim, dim_context = get_encoded_dim(t5_name), cross_attend = has_condition, grad_shrink_alpha = grad_shrink_alpha, **kwargs)
         self.to_logits = nn.Linear(dim, num_semantic_tokens + 1)
 
+    @property
+    def device(self):
+        return next(self.parameters()).device
+
+    @eval_decorator
+    @torch.no_grad()
+    def generate(
+        self,
+        *,
+        max_length,
+        prime_wave = None,
+        prime_ids = None,
+        batch_size = 1,
+        cond_scale = 3,
+        filter_thres = 0.9,
+        temperature = 1.,
+        **kwargs
+    ):
+        device = self.device
+
+        # derive wav2vec ids from the input wave
+
+        if exists(prime_wave):
+            assert not exists(prime_ids)
+            assert exists(self.wav2vec)
+            ids = self.wav2vec(prime_wave, flatten = False)
+        elif exists(prime_ids):
+            ids = prime_ids
+        else:
+            ids = torch.empty((batch_size, 0), dtype = torch.long, device = device)
+
+        if self.unique_consecutive:
+            ids = batch_unique_consecutive(ids, pad_value = self.pad_id)
+
+        # start length and get running id output
+
+        start_length = ids.shape[-1]
+        output = ids.clone()
+
+        # sample from transformer
+
+        for ind in range(start_length, max_length):
+
+            logits = self.forward_with_cond_scale(
+                ids = output,
+                **kwargs
+            )
+
+            last_logits = logits[:, -1]
+            filtered_logits = top_k(last_logits, thres = filter_thres)
+            sampled = gumbel_sample(filtered_logits, temperature = temperature, dim = -1)
+
+            sampled = rearrange(sampled, 'b -> b 1')
+            output = torch.cat((output, sampled), dim = -1)
+
+            if all_rows_have_eos_id(output, self.eos_id):
+                break
+
+        output = mask_out_after_eos_id(output, self.eos_id)
+        return output
+
+    def forward_with_cond_scale(
+        self,
+        *args,
+        cond_scale = 3,
+        **kwargs
+    ):
+        logits = self.forward(*args, cond_drop_prob = 0., **kwargs)
+
+        if cond_scale == 1 or not self.has_condition:
+            return logits
+
+        null_logits = self.forward(*args, cond_drop_prob = 1., **kwargs)
+        return null_logits + (logits - null_logits) * cond_scale
+
     def forward(
         self,
         *,
@@ -311,7 +428,7 @@ class SemanticTransformer(nn.Module):
         text_embeds = None,
         cond_drop_prob = None
     ):
-        device = next(self.parameters()).device
+        device = self.device
 
         assert exists(raw_wave) ^ exists(ids)
 
@@ -321,7 +438,8 @@ class SemanticTransformer(nn.Module):
 
         b = ids.shape[0]
 
-        ids = append_eos_id(ids, self.eos_id)
+        if self.training:
+            ids = append_eos_id(ids, self.eos_id)
 
         if self.unique_consecutive:
             ids = batch_unique_consecutive(ids, pad_value = self.pad_id)
@@ -408,6 +526,24 @@ class CoarseTransformer(nn.Module):
         self.to_semantic_logits = nn.Linear(dim, num_semantic_tokens + 1)
         self.coarse_logit_weights = nn.Parameter(torch.randn(num_coarse_quantizers, codebook_size_with_eos, dim))
 
+    @property
+    def device(self):
+        return next(self.parameters()).device
+
+    def forward_with_cond_scale(
+        self,
+        *args,
+        cond_scale = 3,
+        **kwargs
+    ):
+        logits = self.forward(*args, cond_drop_prob = 0., **kwargs)
+
+        if cond_scale == 1 or not self.has_condition:
+            return logits
+
+        null_logits = self.forward(*args, cond_drop_prob = 1., **kwargs)
+        return null_logits + (logits - null_logits) * cond_scale
+
     def forward(
         self,
         *,
@@ -520,6 +656,24 @@ class FineTransformer(nn.Module):
         self.coarse_logit_weights = nn.Parameter(torch.randn(num_coarse_quantizers, codebook_size_with_eos, dim))
         self.fine_logit_weights = nn.Parameter(torch.randn(num_fine_quantizers, codebook_size_with_eos, dim))
 
+    @property
+    def device(self):
+        return next(self.parameters()).device
+
+    def forward_with_cond_scale(
+        self,
+        *args,
+        cond_scale = 3,
+        **kwargs
+    ):
+        logits = self.forward(*args, cond_drop_prob = 0., **kwargs)
+
+        if cond_scale == 1 or not self.has_condition:
+            return logits
+
+        null_logits = self.forward(*args, cond_drop_prob = 1., **kwargs)
+        return null_logits + (logits - null_logits) * cond_scale
+
     def forward(
         self,
         coarse_token_ids,
